# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import pkg_resources


class SparkContextBuilder:
    def __init__(self):
        pass

    spark_context = None

    @classmethod
    def get_spark_context(cls):
        if cls.spark_context is None:
            from pyspark.sql import SparkSession

            scheduler_path = pkg_resources.resource_filename('webservice', "config/scheduler.xml")

            spark = SparkSession.builder.appName("nexus-analysis").config(
                "spark.scheduler.allocation.file", scheduler_path
            ).config(
                "spark.scheduler.mode", "FAIR"
            ).getOrCreate()
            cls.spark_context = spark.sparkContext

        return cls.spark_context

